﻿// CleanPages.jsx
// An InDesign JavaScript
//
// Cleaning the pages of active document
//
// Copyright: Sho Nuff - shonuff@ya.ru
// License: MIT
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to 
// deal in the Software without restriction, including without limitation the 
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
// sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
// IN THE SOFTWARE.
//
//
// 2012-04-30

#target indesign

//$.locale = 'ru';

if (parseFloat(app.version.split('.')[0]) < 6) {
	alert(localize({
		en: 'This script requires version of InDesign CS4 and more', 
		ru: 'Этот скрипт не работает в версиях InDesign ниже CS4'
	}));
	exit();
}

if (!app.documents.length) exit(); // нет открытых документов

// глобальные переменные
var 
	document = app.activeDocument,
	// script preferences
	prefsFile = new File(Folder.userData.absoluteURI + '/CleanPages.conf' ),
	prefs = {
		startSpreadNumber: 0,
		endSpreadNumber:   document.spreads.length - 1,
		cleanPages:        true,
		cleanPasteboard:   true,
		activeLayers:      '',
		removeTextFrames:  true,
		removePolygons:    true,
		removeGuides:      true,
		removeOther:       true,
		removeLocked:      false,
	};

// прочитаем предыдущие настройки
(function(){
	if (prefsFile.exists && prefsFile.open('r')) {
		
		try {
			var p = eval(prefsFile.read());
			for (var i in p) prefs[i] = p[i];
		} catch (e) {}
		
		prefsFile.close();
	}
})();

// уточним настройки
prefs.startSpreadNumber = Math.min(prefs.startSpreadNumber, document.spreads.length - 1);
prefs.endSpreadNumber =   Math.min(prefs.endSpreadNumber,   document.spreads.length - 1);

// возвращают имена как в палитре Pages
Page.prototype.getName = function() {
	var section = this.appliedSection;
		
	return (section.includeSectionPrefix ? '' :  section.sectionPrefix) + this.name;
};

Spread.prototype.getName = function() {
	var name = this.pages.item(0).getName();
	
	if (this.pages.length > 1) {
		name += ' - ' + this.pages.item(-1).getName();
	} 
	return name;
};

// создаем диалог
(function() {
	while (true) {
		var window =             new Window("dialog");
		window.text =            "Clean Pages";
		window.orientation =     "row";
		var colMain =            window.add("group");
		colMain.orientation =    "column";
		colMain.alignChildren =  "left";
		colMain.minWidth =       280;
		
		// кнопки
		var colButtons =         window.add("group");
		colButtons.orientation = "column";
		colButtons.alignment =   "top";
		colButtons.add("button", undefined, "OK");
		colButtons.add("button", undefined, "Cancel");
		var pnPages = colMain.add("panel");
		pnPages.minimumSize = [colMain.minWidth, 0];
		pnPages.alignChildren = "left";
		var txPages =  pnPages.add("statictext");
		txPages.text = document.documentPreferences.facingPages ? 
			localize({en: 'Spreads:', ru: 'Развороты:'}) :
			localize({en: 'Pages:', ru: 'Страницы:'});
		
		// выбор первой страницы/разворота диапазона
		var rowStartSpread =        pnPages.add("group");
		var txStartSpread =         rowStartSpread.add("statictext");
		txStartSpread.text = localize({en: 'Start from:', ru: 'Начать c:'});
		txStartSpread.minimumSize = [100, 0];
		rowStartSpread.addEventListener("click", function(event) {
			if (!event.target || event.target.constructor.name != 'StaticText') return;
			pnLayers.fl = !pnLayers.fl;
			
			ddStartSpread.selection = 0;
		});
		var ddStartSpread = rowStartSpread.add("dropdownlist");
		var ddEndSpread = null;
		
		// выбор последней страницы/разворота диапазона
		var rowEndPage = pnPages.add("group");
		rowEndPage.addEventListener("click", function(event) {
			if (!event.target || event.target.constructor.name != 'StaticText') return;
			pnLayers.fl = !pnLayers.fl;
			ddEndSpread.selection = ddEndSpread.items.length - 1;
		});
		var txEndPage =          rowEndPage.add("statictext");
		txEndPage.text = localize({en: 'End in:', ru: 'Закончить на:'});
		txEndPage.minimumSize =  [100, 0];
		var ddEndSpread = rowEndPage.add("DropDownList");
		
		
		for (var i = 0, l = document.spreads.length; i < l; i++) {
			ddStartSpread.add("item", document.spreads[i].getName());
			ddEndSpread.add("item", document.spreads[i].getName());
			
			if (i == prefs.startSpreadNumber) ddStartSpread.selection = i;
			if (i == prefs.endSpreadNumber)   ddEndSpread.selection =   i;
		}
		// корректировка StartPage
		ddEndSpread.addEventListener("change", function(event) {
			if (ddStartSpread && ddStartSpread.selection.index > ddEndSpread.selection.index) {
				ddStartSpread.selection = ddEndSpread.selection.index;
			}
		});
		// корректировка EndPage
		ddStartSpread.addEventListener("change", function(event) {
			if (ddEndSpread && ddEndSpread.selection.index < ddStartSpread.selection.index) {
				ddEndSpread.selection = ddStartSpread.selection.index;
			}
		});
		
		// размещение объектов
		var pnLocation =           colMain.add("panel");
		pnLocation.minimumSize =   [colMain.minWidth, 0];
		pnLocation.orientation =   "column";
		pnLocation.alignChildren = "left";
		pnLocation.add("statictext").text = localize({en: 'Location:', ru: 'Размещение объектов:'});
		var cbCleanPages = pnLocation.add("checkbox", undefined, 
			localize({en: 'Page', ru: 'На странице'}));
		cbCleanPages.value = !!prefs.cleanPages;
		var cbCleanPasteboard = pnLocation.add("checkbox", undefined, 
			localize({en: 'Pasteboard', ru: 'На монтажном столе'}));
		cbCleanPasteboard.value = !!prefs.cleanPasteboard;

		// список слоев
		var pnLayers =           colMain.add("panel");
		pnLayers.minimumSize =   [colMain.minWidth, 0];
		pnLayers.orientation =   "column";
		pnLayers.alignChildren = "left";
		pnLayers.add("statictext").text = localize({en: 'Clean Layers:', ru: 'Очистить слои:'});
		
		// включение/отключение всех
		pnLayers.addEventListener("click", function(event) {
			if (!event.target || event.target.constructor.name != 'StaticText') return;
			cbLayers[0].value = !cbLayers[0].value;
			
			for (var i = cbLayers.length; i--; ){
				cbLayers[i].value = cbLayers[0].value;
			}
		});
		var cbLayers = [];
		
		for (var i = 0, l = document.layers.length; i < l; i++) {
			var layerName = document.layers[i].name;
			cbLayers[i] = pnLayers.add(
				"checkbox", 
				undefined,
				layerName
			);
			cbLayers[i].value = prefs.activeLayers.indexOf('\t' + layerName + '\t') >= 0;
		}
		
		// список объектов
		var pnTypes =           colMain.add("panel");
		pnTypes.minimumSize =   [colMain.minWidth, 0];
		pnTypes.orientation =   "column";
		pnTypes.alignChildren = "left";
		pnTypes.add("statictext").text = localize(localize({en:'Remove:', ru: 'Удалять объекты:'}));
		
		// включение/отключение всех
		pnTypes.addEventListener("click", function(event) {
			if (!event.target || event.target.constructor.name != 'StaticText') return;
			cbTextFrames.value = !cbTextFrames.value;
			cbPolygons.value =   cbTextFrames.value;
			cbGuides.value =     cbTextFrames.value;
			cbOther.value =      cbTextFrames.value;
		});
		
		var cbTextFrames = pnTypes.add("checkbox", undefined, 
			localize({en: 'Text Frames', ru: 'Текстовые фреймы'}));
		cbTextFrames.value = !!prefs.removeTextFrames;
		var cbPolygons = pnTypes.add("checkbox", undefined, 
			localize({en: 'Lines, Ovals, Polygons', ru: 'Линии, овалы, многоугольники'}));
		cbPolygons.value = !!prefs.removePolygons;
		var cbGuides = pnTypes.add("checkbox", undefined, 
			localize({en:'Guides', ru: 'Направляющие'}));
		cbGuides.value = !!prefs.removeGuides;
		var cbOther = pnTypes.add("checkbox", undefined, 
			localize({en:'Other', ru: 'Прочие'}));
		cbOther.value = !!prefs.removeOther;
		
		// очистка залоченных
		var cbLocked = colMain.add("checkbox", undefined, 
			localize({en: 'Remove Locked', ru: 'Удалять заблокированные'}));
		cbLocked.value = !!prefs.removeLocked;
		
		// покажем диалог
		if (2 == window.show()) {
			// если нажата кнопка отмены - выход
			exit();
		}
		
		// обновим prefs
		prefs.startSpreadNumber = Math.min(ddStartSpread.selection.index, ddEndSpread.selection.index);
		prefs.endSpreadNumber =   Math.max(ddStartSpread.selection.index, ddEndSpread.selection.index);
		prefs.cleanPages =        cbCleanPages.value;
		prefs.cleanPasteboard =   cbCleanPasteboard.value;
		prefs.removeTextFrames =  cbTextFrames.value;
		prefs.removePolygons =    cbPolygons.value;
		prefs.removeGuides =      cbGuides.value;
		prefs.removeOther =       cbOther.value;
		prefs.removeLocked =      cbLocked.value;
		prefs.spreadsMode =       document.documentPreferences.facingPages;
		prefs.activeLayers =      '\t';
		
		for (var i = cbLayers.length; i--; ) {
			if (cbLayers[i].value) prefs.activeLayers += cbLayers[i].text + '\t';
		}
		if (prefs.activeLayers.length < 2) {
			alert(localize({en: 'Select Layer, please', ru: 'Укажите хотя бы один слой'}));
			continue;
		}
		if (!(prefs.removeTextFrames || prefs.removePolygons || 
			prefs.removeGuides || prefs.removeOther )) {
			
			alert(localize({
				en: 'Choice Type of the Objects', 
				ru: 'Не выбраны объекты для удаления'
			}));
			continue;
		}
		if (!prefs.cleanPages && !prefs.cleanPasteboard) {
			
			alert(localize({
				en: 'Choice Location of the Objects', 
				ru: 'Укажите размещение объектов'
			}));
			continue;
		}
		break;
	}
})();
// конец диалога

// сохраним prefs
if (!prefsFile.open('w') || !prefsFile.write(prefs.toSource())) {
	alert(localise({en: 'Error. Can not save settings', ru: 'Ошибка\nНе удалось сохранить настройки.'}));
}
prefsFile.close();

// обходит коллекцию pageItems и удаляет подходящие по параметрам элементы
function CleanItems(items, itemsInPage) {
	itemsInPage = itemsInPage || '';
	seekItem:
	for (var i = items.length; i--; ) {
		// возможно элемент уже удален
		if (!items[i] || !items[i].isValid) continue;
		
		var item = items[i];
		
		// пропустим залоченные
		if (!prefs.removeLocked && item.locked) continue;

		// проверим слой
		if (item.itemLayer.locked) continue;
		if (prefs.activeLayers.indexOf('\t' + item.itemLayer.name + '\t') < 0) continue;
		
		if (location == 'Page' && (prefs.onlyFromSpreads || 
			!pageInSelectedRange([page]))) continue;
		
		// проверим конструктор
		switch (item.constructor.name) {
			case 'Group':
				continue seekItem;
				
			case 'TextFrame':
				if (!prefs.removeTextFrames) continue seekItem;
				break;
			
			case 'GraphicLine':
			case 'Oval':
			case 'Polygon':
			case 'Rectangle':
				if (!prefs.removePolygons) continue seekItem;
				break;
			
			case 'Guide':
				if (!prefs.removeGuides ||
					(!prefs.cleanPages && item.fitToPage) ||
					(!prefs.cleanPasteboard && !item.fitToPage)) continue seekItem;
					
				item.locked && item.locked = false;
				item.remove();
				continue seekItem;
				break;
			
			default: 
				if (!prefs.removeOther) continue seekItem;
		}
		// найдем и проверим страницу
		var page = item;
		
		do {
			var group = page;
			page = page.parent;
			var location = page.constructor.name;
			
			if (location == 'Application') continue seekItem;
		} while (location != 'Page' && location != 'Spread');
		
		var inPage = itemsInPage.indexOf(' ' + group.id + ' ') >= 0;
		
		if (inPage && !prefs.cleanPages) continue;
		if (!inPage && !prefs.cleanPasteboard) continue;
	
		// удаляем
		var groupLocked = group.locked;
		groupLocked && group.locked = false;
		item.remove();
		groupLocked && group.isValid && group.locked = groupLocked;
	}
}

// обернем основные операции в одну функцию 
// чтобы все действия скрипта можно было отменить одной командой
function CleanPages() {
	var unlockedLayers = [];
		
	// разлочим все слои
	if (prefs.removeLocked) {
		for (var i = document.layers.length; i--; ) {
			var layer = document.layers[i];
			
			if (layer.locked) {
				unlockedLayers.push(layer);
				layer.locked = false;
			}
		}
	}

	// обход разворотов
	for (var i = prefs.startSpreadNumber; i <= prefs.endSpreadNumber; i++) {
		var spread = document.spreads[i];
		
		// список объектов размещенных на страницах
		var itemsInPage = ' ' + 
			spread.pages.everyItem().pageItems.everyItem().id.join(' ') + ' ';
		
		CleanItems(spread.allPageItems, itemsInPage);
		prefs.removeGuides && CleanItems(spread.guides);
	}
	
	// залочим обратно слои
	for (var i = unlockedLayers.length; i--; ) {
		unlockedLayers[i].locked = true;
	}
}
// вызовем CleanPages 
app.doScript(
	CleanPages, 
	undefined, 
	undefined, 
	UndoModes.FAST_ENTIRE_SCRIPT, 
	'Clean Pages'
);
